#include <ros/ros.h>
#include <geometry_msgs/PoseStamped.h>
#include <tf/transform_listener.h>

#include <cmath>
#include <iostream>
#include <fstream>
#include <sstream>

int main(int argc, char **argv) {

    ros::init(argc, argv, "auto_wp_setter");
    ros::NodeHandle node("~");
    tf::TransformListener listener;


    //Waypoint を設定する間隔
    double wp_tolerance_dist_m;
    double wp_tolerance_angle_deg;

    node.param("d_tolerance", wp_tolerance_dist_m, 1.0);
    node.param("a_tolerance", wp_tolerance_angle_deg, 30.0);

    std::ofstream wp_file("wp.txt");


    // 固定小数点記法に設定
    wp_file << std::fixed;

    double pre_wp_x = 0.0, pre_wp_y = 0.0, pre_wp_yaw = 0.0;

    while (ros::ok())
    {
        tf::StampedTransform transform;
        try{
//                ros::Time current_time = ros::Time::now();
            listener.waitForTransform(
                        "/map", "/base_link",
                        ros::Time(0),
//                            ros::Time(0) + ros::Duration().fromSec(0.5),
                        ros::Duration(1.0));
            listener.lookupTransform("/map", "/base_link", ros::Time(0), transform);
        } catch (tf::TransformException ex){
            ROS_ERROR("%s", ex.what());
        }


        double diff_x = pre_wp_x - transform.getOrigin().x();
        double diff_y = pre_wp_y - transform.getOrigin().y();
        double range_m_sq = diff_x*diff_x + diff_y*diff_y;
        double diff_yaw_rad = fabs(pre_wp_yaw - tf::getYaw(transform.getRotation()));
        while(diff_yaw_rad >= M_PI) diff_yaw_rad -=2*M_PI;
        while(diff_yaw_rad <= -M_PI) diff_yaw_rad +=2*M_PI;

        std::cout<< sqrt(range_m_sq) << "[m]" <<std::endl;

        if (range_m_sq > wp_tolerance_dist_m * wp_tolerance_dist_m || diff_yaw_rad > wp_tolerance_angle_deg/180*M_PI)
        {
            wp_file << transform.getOrigin().x() << "\t"
                    << transform.getOrigin().y() << "\t"
                    << tf::getYaw(transform.getRotation()) << std::endl;

            pre_wp_x = transform.getOrigin().x();
            pre_wp_y = transform.getOrigin().y();
            pre_wp_yaw = tf::getYaw(transform.getRotation());
        }

        ros::Duration(0.1).sleep();

    }


    ros::spinOnce();

    return 0;
}
